﻿<#
.SYNOPSIS
	This script performs the installation or uninstallation of an application(s).
	# LICENSE #
	PowerShell App Deployment Toolkit - Provides a set of functions to perform common application deployment tasks on Windows.
	Copyright (C) 2017 - Sean Lillis, Dan Cunningham, Muhammad Mashwani, Aman Motazedian.
	This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	You should have received a copy of the GNU Lesser General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
.DESCRIPTION
	The script is provided as a template to perform an install or uninstall of an application(s).
	The script either performs an "Install" deployment type or an "Uninstall" deployment type.
	The install deployment type is broken down into 3 main sections/phases: Pre-Install, Install, and Post-Install.
	The script dot-sources the AppDeployToolkitMain.ps1 script which contains the logic and functions required to install or uninstall an application.
.PARAMETER DeploymentType
	The type of deployment to perform. Default is: Install.
.PARAMETER DeployMode
	Specifies whether the installation should be run in Interactive, Silent, or NonInteractive mode. Default is: Interactive. Options: Interactive = Shows dialogs, Silent = No dialogs, NonInteractive = Very silent, i.e. no blocking apps. NonInteractive mode is automatically set if it is detected that the process is not user interactive.
.PARAMETER AllowRebootPassThru
	Allows the 3010 return code (requires restart) to be passed back to the parent process (e.g. SCCM) if detected from an installation. If 3010 is passed back to SCCM, a reboot prompt will be triggered.
.PARAMETER TerminalServerMode
	Changes to "user install mode" and back to "user execute mode" for installing/uninstalling applications for Remote Destkop Session Hosts/Citrix servers.
.PARAMETER DisableLogging
	Disables logging to file for the script. Default is: $false.
.EXAMPLE
    powershell.exe -Command "& { & '.\Deploy-Application.ps1' -DeployMode 'Silent'; Exit $LastExitCode }"
.EXAMPLE
    powershell.exe -Command "& { & '.\Deploy-Application.ps1' -AllowRebootPassThru; Exit $LastExitCode }"
.EXAMPLE
    powershell.exe -Command "& { & '.\Deploy-Application.ps1' -DeploymentType 'Uninstall'; Exit $LastExitCode }"
.EXAMPLE
    Deploy-Application.exe -DeploymentType "Install" -DeployMode "Silent"
.NOTES
	Toolkit Exit Code Ranges:
	60000 - 68999: Reserved for built-in exit codes in Deploy-Application.ps1, Deploy-Application.exe, and AppDeployToolkitMain.ps1
	69000 - 69999: Recommended for user customized exit codes in Deploy-Application.ps1
	70000 - 79999: Recommended for user customized exit codes in AppDeployToolkitExtensions.ps1
.LINK
	http://psappdeploytoolkit.com
#>
[CmdletBinding()]
Param (
    [Parameter(Mandatory = $false)]
    [ValidateSet('Install', 'Uninstall', 'Repair')]
    [string]$DeploymentType = 'Install',
    [Parameter(Mandatory = $false)]
    [ValidateSet('Interactive', 'Silent', 'NonInteractive')]
    [string]$DeployMode = 'Interactive',
    [Parameter(Mandatory = $false)]
    [switch]$AllowRebootPassThru = $false,
    [Parameter(Mandatory = $false)]
    [switch]$TerminalServerMode = $false,
    [Parameter(Mandatory = $false)]
    [switch]$DisableLogging = $false
)

Try
{
    ## Set the script execution policy for this process
    Try { Set-ExecutionPolicy -ExecutionPolicy 'ByPass' -Scope 'Process' -Force -ErrorAction 'Stop' }
    Catch { }
    
    ##*===============================================
    ##* VARIABLE DECLARATION
    ##*===============================================
    ## Variables: Application
    [string]$appVendor = 'Cisco'
    [string]$appName = 'Cisco secure client'
    [string]$appVersion = '5.1.2.42'
    [string]$appArch = ''
    [string]$appTarget = 'ENT'
    [string]$appLang = 'ENG'
    [string]$appRevision = 'A'
    [string]$appScriptVersion = '1.0.0'
    [string]$appScriptDate = '17-March-2023'
    [string]$appScriptAuthor = 'Ranjini Janardhanan'
    [string]$ESDNumber = 'RITM8882297'
    [string]$ESDRequestor = 'Karthik Rajendran'
    [BOOL]$RebootRequired = $true #-$true or $false
    
    ## Keep notes below on what changes were made if $appRevision changes
    # Version B - 
    
    [string]$StartTime = Get-Date -Format g
    [string]$Status = ''
    [string]$FINALMSG = ''
    ##*===============================================
    ## Variables: Install Titles (Only set here to override defaults set by the toolkit)
    #[string]$installName = ''
    [string]$installTitle = 'Cisco secure client 5.1.2.42 Client - Azure'
    
    ##* Do not modify section below
    #region DoNotModify
    
    ## Variables: Exit Code
    [int32]$mainExitCode = 0
    
    ## Variables: Script
    [string]$deployAppScriptFriendlyName = 'Deploy Application'
    [version]$deployAppScriptVersion = [version]'3.8.1'
    [string]$deployAppScriptDate = '28/03/2020'
    [hashtable]$deployAppScriptParameters = $psBoundParameters
    
    ## Variables: Environment
    If (Test-Path -LiteralPath 'variable:HostInvocation') { $InvocationInfo = $HostInvocation }
    Else { $InvocationInfo = $MyInvocation }
    [string]$scriptDirectory = Split-Path -Path $InvocationInfo.MyCommand.Definition -Parent
    
    ## Dot source the required App Deploy Toolkit Functions
    Try
    {
        [string]$moduleAppDeployToolkitMain = "$scriptDirectory\AppDeployToolkit\AppDeployToolkitMain.ps1"
        If (-not (Test-Path -LiteralPath $moduleAppDeployToolkitMain -PathType 'Leaf')) { Throw "Module does not exist at the specified location [$moduleAppDeployToolkitMain]." }
        If ($DisableLogging) { . $moduleAppDeployToolkitMain -DisableLogging }
        Else { . $moduleAppDeployToolkitMain }
    }
    Catch
    {
        If ($mainExitCode -eq 0) { [int32]$mainExitCode = 60008 }
        Write-Error -Message "Module [$moduleAppDeployToolkitMain] failed to load: `n$($_.Exception.Message)`n `n$($_.InvocationInfo.PositionMessage)" -ErrorAction 'Continue'
        ## Exit the script, returning the exit code to SCCM
        If (Test-Path -LiteralPath 'variable:HostInvocation') { $script:ExitCode = $mainExitCode; Exit }
        Else { Exit $mainExitCode }
    }
    
    #endregion
    ##* Do not modify section above
    [string]$Attempts = Create-HWCustomRegistryFields
    ##*===============================================
    ##* END VARIABLE DECLARATION
    ##*===============================================
    
    If ($deploymentType -ine 'Uninstall' -and $deploymentType -ine 'Repair')
    {
        ##*===============================================
        ##* PRE-INSTALLATION
        ##*===============================================
        [string]$installPhase = 'Pre-Installation'
        
        Show-InstallationWelcome -AllowDefer -DeferTimes 1 -CheckDiskSpace
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    GET DETAILS OF CURRENT ANYCONNECT INSTALLED" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        $AnyConnectInstalled = (Get-InstalledApplication -name 'Cisco AnyConnect Secure Mobility Client')
             
 
        ##*===============================================
        ##* INSTALLATION
        ##*===============================================
        [string]$installPhase = 'Installation'
        
        ## Handle Zero-Config MSI Installations
        If ($useDefaultMsi)
        {
            [hashtable]$ExecuteDefaultMSISplat = @{ Action = 'Install'; Path = $defaultMsiFile }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
            Execute-MSI @ExecuteDefaultMSISplat; If ($defaultMspFiles) { $defaultMspFiles | ForEach-Object { Execute-MSI -Action 'Patch' -Path $_ } }
        }
        
        Show-InstallationProgress
        
        $UPDATE_VERSION = '5.1.2.42'
        
        $ANYCONNECT_WIN = 'cisco-secure-client-win-5.1.2.42-core-vpn-predeploy-k9.msi'
        $ANYCONNECT_UMB = 'cisco-secure-client-win-5.1.2.42-umbrella-predeploy-k9.msi'
        $ANYCONNECT_DART = 'cisco-secure-client-win-5.1.2.42-dart-predeploy-k9.msi'
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    INSTALL VPN -  Install the module that enables VPN capabilities  " -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        Execute-MSI -Action 'Install' -Path "$dirFiles\$ANYCONNECT_WIN"
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    VERIFY SUCCESSFUL INSTALL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        $vpnInstalled = (Get-InstalledApplication -name 'Cisco Secure Client - AnyConnect VPN')
        $vpnVersion = ($vpnInstalled).DisplayVersion
        
        if (($vpnInstalled) -and ($vpnVersion -ge "$UPDATE_VERSION"))
        {
            Write-Log -Message "VPN INSTALL SUCCESSFUL." -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        else
        {
            $FINALMSG = "VPN INSTALL FAILED."
            $Status = "Failed"
            Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
            Exit-Script -ExitCode 1
        }
               
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    INSTALL UMBRELLA" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        Execute-MSI -Action 'Install' -Path "$dirFiles\$ANYCONNECT_UMB"
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    VERIFY SUCCESSFUL INSTALL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        $umbInstalled = (Get-InstalledApplication -name 'Cisco Secure Client - Umbrella')
        $umbVersion = ($umbInstalled).DisplayVersion
        
        if (($umbInstalled) -and ($umbVersion -ge "$UPDATE_VERSION"))
        {
            Write-Log -Message "UMBRELLA INSTALL SUCCESSFUL." -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        else
        {
            $FINALMSG = "UMBRELLA INSTALL FAILED."
            $Status = "Failed"
            Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
            Exit-Script -ExitCode 1
        }
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    INSTALL DART - Installs the diagnostics module that collects AnyConnect Secure Mobility Client troubleshooting information" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        Execute-MSI -Action 'Install' -Path "$dirFiles\$ANYCONNECT_DART"
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    VERIFY SUCCESSFUL INSTALL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        $dartInstalled = (Get-InstalledApplication -name 'Cisco Secure Client - Diagnostics and Reporting Tool')
        $dartVersion = ($dartInstalled).DisplayVersion
        
        if (($dartInstalled) -and ($dartVersion -ge "$UPDATE_VERSION"))
        {
            Write-Log -Message "DART INSTALL SUCCESSFUL." -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        else
        {
            $FINALMSG = "DART INSTALL FAILED."
            $Status = "Failed"
            Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
            Exit-Script -ExitCode 1
        }
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message '    BACKUP PREVIOUS ANYCONNECT XML FILES. COPY NEW FILES' -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        $Profile_Path = (Test-Path -Path "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile")
        Write-Log -Source $deployAppScriptFriendlyName -Message "DEBUG: Check for existence of $env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile = $Profile_Path"
        if ($Profile_Path -eq $true)
        {
            New-Folder -Path "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile\backup"
            Copy-File -Path "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile\*.*" -Destination "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile\backup"
            Remove-File -Path "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile\*.xml"
            Copy-File -Path "$dirFiles\Profiles\vpn\Honeywell-Azure.xml" -Destination "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile\Honeywell-Azure.xml"
        }
        else
        {
            New-Folder -Path "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile"
            Copy-File -Path "$dirFiles\Profiles\vpn\Honeywell-Azure.xml" -Destination "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile\Honeywell-Azure.xml"
        }
        
        ## Modify ACL on Profile folder
        #first load the ACL
        $folder_to_modify = "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Profile"
        $acl = Get-Acl "$folder_to_modify"
        $Ar = New-Object System.Security.AccessControl.FileSystemAccessRule("Users", "FullControl", "ContainerInherit,ObjectInherit", "None", "Allow")
        $acl.SetAccessRule($ar)
        #re-write the acl on the target file
        Set-Acl "$folder_to_modify" $acl
  
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    COPY UMBRELLA CONFIG FILE" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Copy-File -Path "$dirFiles\Profiles\umbrella\OrgInfo.json" -Destination "$env:ProgramData\Cisco\Cisco AnyConnect Secure Mobility Client\Umbrella\OrgInfo.json"
            
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "    FIREWALL SETTINGS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        Set-RegistryKey -Key "HKLM\SYSTEM\CurrentControlSet\Services\mpssvc" -Name "Start" -Value "2" -Type DWord -ContinueOnError:$True
        Set-RegistryKey -Key "HKLM\SOFTWARE\Policies\Microsoft\WindowsFirewall\DomainProfile" -Name "EnableFirewall" -Value "1" -Type DWord -ContinueOnError:$True
        Set-RegistryKey -Key "HKLM\SOFTWARE\Policies\Microsoft\WindowsFirewall\StandardProfile" -Name "EnableFirewall" -Value "1" -Type DWord -ContinueOnError:$True
        Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        

                ## Import a certificate to system 'Trusted Root Certification Authorities\Certificates' store.
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***IMPORT CERTIFICATE***" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        if ([environment]::OSVersion.Version.Major -eq 10)
        {
            Import-Certificate -FilePath "$dirFiles\Cisco_Umbrella_Root_CA.cer" -CertStoreLocation 'Cert:\LocalMachine\Root'
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "    VERIFY SUCCESSFUL INSTALL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            $certInstalled = Get-ChildItem -Path Cert:\LocalMachine\Root | Where-Object {$_.Thumbprint -eq "c5091132e9adf8ad3e33932ae60a5c8fa939e824"}   
            if ($certInstalled)
            {
                Write-Log -Message "Imported Cert Cisco_Umbrella_Root_CA.cer" -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "Certificate import failed."
                $Status = 'Failed'
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Exit-Script -ExitCode 1
            }
        }
        
        if ([environment]::OSVersion.Version.Major -eq 6)
        {
            Execute-Process -Path "certutil.exe" -Parameters "-f -addstore -enterprise root `"$dirFiles\Cisco_Umbrella_Root_CA.cer`""
            
            if ($is64Bit -eq $true)
            {
                $certInstalled = Get-ChildItem -Path Cert:\LocalMachine\Root | Where-Object {$_.Thumbprint -eq "c5091132e9adf8ad3e33932ae60a5c8fa939e824"}
            }
            else
            {
                $certInstalled = Get-ChildItem -Path Cert:\LocalMachine\Root | Where-Object {$_.Thumbprint -eq "c5091132e9adf8ad3e33932ae60a5c8fa939e824"}
                
            }
            if ($certInstalled)
            {
                Write-Log -Message "Imported Cert Cisco_Umbrella_Root_CA.cer" -Source $deployAppScriptFriendlyName
            }
            else
            {
                $FINALMSG = "Certificate import failed."
                $Status = 'Failed'
                Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
                Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
                Exit-Script -ExitCode 1
            }
        }
        


        ##*===============================================
        ##* POST-INSTALLATION
        ##*===============================================
        [string]$installPhase = 'Post-Installation'
        
        ## <Perform Post-Installation tasks here>
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Update-HWCustomRegistryFields -Status "Success" -FINALMSG "Successful Installation" -ExitCode $returnCode
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "RUN HW INVENTORY POLICY" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Invoke-SCCMTask -ScheduleID 'HardwareInventory'
        #Invoke-SCCMTask -ScheduleID 'ApplicationManagerPolicyAction'
        Write-Log -Message "*" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        
        If ($RebootRequired) { Show-InstallationRestartPrompt -NoCountdown }
        Show-InstallationRestartPrompt -Countdownseconds 28800 -CountdownNoHideSeconds 60
        ## Display a message at the end of the install
        #If (-not $useDefaultMsi) { Show-InstallationPrompt -Message "$installTitle installed successfully" -ButtonRightText 'OK' -Icon Information -NoWait }
    }
    ElseIf ($deploymentType -ieq 'Uninstall')
    {
        ##*===============================================
        ##* PRE-UNINSTALLATION
        ##*===============================================
        [string]$installPhase = 'Pre-Uninstallation'
        
        ## Show Welcome Message, for example to close Internet Explorer type iexplore with a 60 second countdown before automatically closing
        #Show-InstallationWelcome -CloseApps 'appname without exe' -CloseAppsCountdown 60
        
        ## Show Progress Message (with the default message)
        Show-InstallationProgress
        
        
        ##*===============================================
        ##* UNINSTALLATION
        ##*===============================================
        [string]$installPhase = 'Uninstallation'
        
        ## Handle Zero-Config MSI Uninstallations
        If ($useDefaultMsi)
        {
            [hashtable]$ExecuteDefaultMSISplat = @{ Action = 'Uninstall'; Path = $defaultMsiFile }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
            Execute-MSI @ExecuteDefaultMSISplat
        }
        
        Remove-MSIApplications -Name "Cisco Secure Client - Umbrella"
        $umbInstalled = (Get-InstalledApplication -name 'Cisco Secure Client - Umbrella')
        if (!$umbInstalled)
        {
            Write-Log -Message "UMBRELLA UNINSTALL SUCCESSFUL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        else
        {
            $FINALMSG = "UMBRELLA UNINSTALL FAILED."
            $Status = 'Failed'
            $installStatus = 'Failed'
            
            Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
            
        }
        
        
       
        
        Remove-MSIApplications -Name "Cisco Secure Client - AnyConnect VPN"
        $vpnInstalled = (Get-InstalledApplication -name 'Cisco Secure Client - AnyConnect VPN')
        if (!$vpnInstalled)
        {
            Write-Log -Message "VPN UNINSTALL SUCCESSFUL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        else
        {
            $FINALMSG = "VPN UNINSTALL FAILED."
            $Status = 'Failed'
            $installStatus = 'Failed'
            
            Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
            
        }

        
        Remove-MSIApplications -Name "Cisco Secure Client - Diagnostics and Reporting Tool"
        $dartInstalled = (Get-InstalledApplication -name 'Cisco Secure Client - Diagnostics and Reporting Tool')
        if (!$dartInstalled)
        {
            Write-Log -Message "DART UNINSTALL SUCCESSFUL" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        else
        {
            $FINALMSG = "DART UNINSTALL FAILED."
            $Status = 'Failed'
            $installStatus = 'Failed'
          
            Write-Log -Message "$FINALMSG" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
        }
        
        if ($installStatus -eq 'FAILED')
        {
            Write-Log -Message "ANYCONNECT UNINSTALL FAILED." -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
            Update-HWCustomRegistryFields -Status "$Status" -FINALMSG "$FINALMSG" -ExitCode $returnCode
            Show-InstallationPrompt -Message "All or part of the AnyConnect uninstall failed." -ButtonRightText 'OK' -Icon Information -NoWait
            Exit-Script -ExitCode 1
        }
        else
        {
            Write-Log -Message "AnyConnect uninstall successful" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        }
        
        ##*===============================================
        ##* POST-UNINSTALLATION
        ##*===============================================
        [string]$installPhase = 'Post-Uninstallation'
        
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "WRITE TO HW CUSTOM REG KEYS" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
        Remove-HWCustomRegistryFields
        Update-HWCustomRegistryFields -Status "Success" -FINALMSG "Successful Uninstall" -ExitCode $returnCode
        
        If ($RebootRequired) { Show-InstallationRestartPrompt -NoCountdown }
        Show-InstallationRestartPrompt -Countdownseconds 28800 -CountdownNoHideSeconds 60
        #Show-InstallationPrompt -Message "$installTitle uninstalled successfully"  -ButtonRightText 'OK' -Icon Information -NoWait
        
        
        
    }
    ElseIf ($deploymentType -ieq 'Repair')
    {
        ##*===============================================
        ##* PRE-REPAIR
        ##*===============================================
        [string]$installPhase = 'Pre-Repair'
        
        ## Show Progress Message (with the default message)
        Show-InstallationProgress
        
        ## <Perform Pre-Repair tasks here>
        
        ##*===============================================
        ##* REPAIR
        ##*===============================================
        [string]$installPhase = 'Repair'
        
        ## Handle Zero-Config MSI Repairs
        If ($useDefaultMsi)
        {
            [hashtable]$ExecuteDefaultMSISplat = @{ Action = 'Repair'; Path = $defaultMsiFile; }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
            Execute-MSI @ExecuteDefaultMSISplat
        }
        # <Perform Repair tasks here>
        
        ##*===============================================
        ##* POST-REPAIR
        ##*===============================================
        [string]$installPhase = 'Post-Repair'
        
        ## <Perform Post-Repair tasks here>
        
        
    }
    ##*===============================================
    ##* END SCRIPT BODY
    ##*===============================================
    
    ## Call the Exit-Script function to perform final cleanup operations
    Exit-Script -ExitCode $mainExitCode
}
Catch
{
    
    
    Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
    Write-Log -Message "AN ERROR WAS DETECTED" -LogType 'CMTrace' -Severity 3 -Source $deployAppScriptFriendlyName
    Write-Log -Message "***************************************************************************************" -LogType 'CMTrace' -Source $deployAppScriptFriendlyName
    
    [int32]$mainExitCode = 60001
    [string]$mainErrorMessage = "$(Resolve-Error)"
    Write-Log -Message $mainErrorMessage -Severity 3 -Source $deployAppScriptFriendlyName
    Update-HWCustomRegistryFields -Status "FAILED" -FINALMSG "AN ERROR WAS DETECTED. CHECK SWD LOG" -ExitCode $mainExitCode
    #    if (!$CurrentLoggedOnUserSession)
    #    {
    #        Show-DialogBox -Text $mainErrorMessage -Icon 'Stop'
    #    }
    Exit-Script -ExitCode $mainExitCode
}


# SIG # Begin signature block
# MIINXAYJKoZIhvcNAQcCoIINTTCCDUkCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUk1OUbuylJa3Ll5U+jNVw/ll5
# rnSgggqeMIIFMDCCBBigAwIBAgIQBAkYG1/Vu2Z1U0O1b5VQCDANBgkqhkiG9w0B
# AQsFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMTMxMDIyMTIwMDAwWhcNMjgxMDIyMTIwMDAwWjByMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQg
# Q29kZSBTaWduaW5nIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# +NOzHH8OEa9ndwfTCzFJGc/Q+0WZsTrbRPV/5aid2zLXcep2nQUut4/6kkPApfmJ
# 1DcZ17aq8JyGpdglrA55KDp+6dFn08b7KSfH03sjlOSRI5aQd4L5oYQjZhJUM1B0
# sSgmuyRpwsJS8hRniolF1C2ho+mILCCVrhxKhwjfDPXiTWAYvqrEsq5wMWYzcT6s
# cKKrzn/pfMuSoeU7MRzP6vIK5Fe7SrXpdOYr/mzLfnQ5Ng2Q7+S1TqSp6moKq4Tz
# rGdOtcT3jNEgJSPrCGQ+UpbB8g8S9MWOD8Gi6CxR93O8vYWxYoNzQYIH5DiLanMg
# 0A9kczyen6Yzqf0Z3yWT0QIDAQABo4IBzTCCAckwEgYDVR0TAQH/BAgwBgEB/wIB
# ADAOBgNVHQ8BAf8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwMweQYIKwYBBQUH
# AQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYI
# KwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFz
# c3VyZWRJRFJvb3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmw0
# LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaG
# NGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RD
# QS5jcmwwTwYDVR0gBEgwRjA4BgpghkgBhv1sAAIEMCowKAYIKwYBBQUHAgEWHGh0
# dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwCgYIYIZIAYb9bAMwHQYDVR0OBBYE
# FFrEuXsqCqOl6nEDwGD5LfZldQ5YMB8GA1UdIwQYMBaAFEXroq/0ksuCMS1Ri6en
# IZ3zbcgPMA0GCSqGSIb3DQEBCwUAA4IBAQA+7A1aJLPzItEVyCx8JSl2qB1dHC06
# GsTvMGHXfgtg/cM9D8Svi/3vKt8gVTew4fbRknUPUbRupY5a4l4kgU4QpO4/cY5j
# DhNLrddfRHnzNhQGivecRk5c/5CxGwcOkRX7uq+1UcKNJK4kxscnKqEpKBo6cSgC
# PC6Ro8AlEeKcFEehemhor5unXCBc2XGxDI+7qPjFEmifz0DLQESlE/DmZAwlCEIy
# sjaKJAL+L3J+HNdJRZboWR3p+nRka7LrZkPas7CM1ekN3fYBIM6ZMWM9CBoYs4Gb
# T8aTEAb8B4H6i9r5gkn3Ym6hU/oSlBiFLpKR6mhsRDKyZqHnGKSaZFHvMIIFZjCC
# BE6gAwIBAgIQBETKus8f9sgLpryqQAAN1TANBgkqhkiG9w0BAQsFADByMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29k
# ZSBTaWduaW5nIENBMB4XDTIxMDMyNjAwMDAwMFoXDTI0MDQwMzIzNTk1OVowgaMx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpOZXcgSmVyc2V5MRYwFAYDVQQHEw1Nb3Jy
# aXMgUGxhaW5zMSUwIwYDVQQKExxIb25leXdlbGwgSW50ZXJuYXRpb25hbCBJbmMu
# MRkwFwYDVQQLExBFbmQgVXNlciBDb21wdXRlMSUwIwYDVQQDExxIb25leXdlbGwg
# SW50ZXJuYXRpb25hbCBJbmMuMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEAl5w/RnE/i8OaoYiqrDsqzAfAM5BaoN5ZByqlKK7BCYKVMHVHtWCRAwNqdwgd
# xI8FnbnV2mkK5SCltKZ9M5n6kbQj2gljatk5rql/cGAd+5BKZIP+AMGZnxfMawjH
# 9VyT0iPv1gGovsavpqzlzQ2Jg/aaD18MtujTIMpAq+XbpJ3ATP9Cx83AxZUSdePd
# S/nXP5YO0TEiqvRcchjAihHaHBDqIsRGAEYWzVtQIWjuZZS0KQrvbfuWm0dqN54X
# MPq1E4R2FilW8/1j4tRzCUnz42axfSz0Tim4JEjB5oRFUuK24eB8AebOH/tLFtJ3
# tSLEYGrdHCB0/cv+EitItd0HEQIDAQABo4IBxDCCAcAwHwYDVR0jBBgwFoAUWsS5
# eyoKo6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFOskOSobl7kny+H5bR9VKmUsDYck
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBu
# MDWgM6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNz
# LWcxLmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNz
# dXJlZC1jcy1nMS5jcmwwSwYDVR0gBEQwQjA2BglghkgBhv1sAwEwKTAnBggrBgEF
# BQcCARYbaHR0cDovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAgGBmeBDAEEATCBhAYI
# KwYBBQUHAQEEeDB2MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5j
# b20wTgYIKwYBBQUHMAKGQmh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdp
# Q2VydFNIQTJBc3N1cmVkSURDb2RlU2lnbmluZ0NBLmNydDAMBgNVHRMBAf8EAjAA
# MA0GCSqGSIb3DQEBCwUAA4IBAQA4OGiu9sPKTUJ4C/oFexJ1y5CUgwMqvj9FV/zs
# AhLp5BnTHxvNVnrid0pQSzu+Ad+QjLcghMaa0aTjDaG+/87c/hi62TFNjibc6HoM
# 4JGNdS6mShEAyM9/gLNDRpc41MYIFrO5Z/sZc1WwTtB23UkLcF4M1eFwsZt5vxtl
# DC79E/rckvC+22Se21wI+hacf0gR2lbgBP7nfEpoB4vjIXTHldfN3wvTl1cS5F5i
# fDoQPdfWYdr77FVJT5LELX1wDoNmI5JdW9OnzP9fJNzXrShzht+XwMWhTfODHuzP
# Cfp4DCqw+VdDKvIAv9AFULNGHdh/jfAd1dnHx2gP2pF5UW7MMYICKDCCAiQCAQEw
# gYYwcjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UE
# CxMQd3d3LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1
# cmVkIElEIENvZGUgU2lnbmluZyBDQQIQBETKus8f9sgLpryqQAAN1TAJBgUrDgMC
# GgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG9w0BCQMxDAYK
# KwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG
# 9w0BCQQxFgQUKl8OnlL2yncmuwJ1820+XakP9QkwDQYJKoZIhvcNAQEBBQAEggEA
# Us+1n59T+u4p2KGWAkeBVKUPC4sx1789yV7aHedeNuhsAGHXXngjw3QNfKJi0fFk
# uxIFoo7Ymqdg4fJXQeIizFzbrXiZ/iv5hUT8WvWFGhm1tkHl4ox7BhJJRWlUtibj
# JMe18ZMVsXJdTUkv9Jtq5BZNB+Ev6vQInozOw5kyl01yAd/BUPBh0cN9pe9ncoGx
# 3TMvtg2OaOsK242aiZMNkpYtWAYWXcPMKXG/fI0FsQr2dB7Q8+bV8Oiw8PskYtWC
# E8w9iGUP5XIK7OGTkcTjMqCN+9vdp/nwXOWWaCVkMHH3FZwSpAYOOTRRBk50IBhn
# RwyQ4MZ6cVErDNS9kAxxlw==
# SIG # End signature block
